package betterwithmods.common.blocks.mechanical;

import betterwithmods.common.BWMBlocks;
import betterwithmods.common.blocks.BWMBlock;
import betterwithmods.util.DirUtils;
import net.minecraft.block.material.Material;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.NonNullList;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import javax.annotation.Nonnull;

import static betterwithmods.util.DirUtils.AXIS;
import static net.minecraft.util.EnumFacing.Axis.Y;

public abstract class BlockAxleGenerator extends BWMBlock implements IBlockActive {

    private static final AxisAlignedBB X_AABB = new AxisAlignedBB(0.0F, 0.375F, 0.375F, 1.0F, 0.625F, 0.625F);
    private static final AxisAlignedBB Y_AABB = new AxisAlignedBB(0.375F, 0.0F, 0.375F, 0.625F, 1.0F, 0.625F);
    private static final AxisAlignedBB Z_AABB = new AxisAlignedBB(0.375F, 0.375F, 0.0F, 0.625F, 0.625F, 1.0F);

    public BlockAxleGenerator(Material material) {
        super(material);
        this.func_180632_j(this.field_176227_L.func_177621_b().func_177226_a(AXIS, EnumFacing.Axis.Z).func_177226_a(ACTIVE, false));
    }

    @Nonnull
    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, AXIS, ACTIVE);
    }

    @Override
    public int func_176201_c(IBlockState state) {
        int axis = state.func_177229_b(AXIS).ordinal();
        int active = state.func_177229_b(ACTIVE) ? 1 : 0;
        return axis | active << 2;
    }

    @Nonnull
    @Override
    public IBlockState func_176203_a(int meta) {
        return func_176223_P().func_177226_a(AXIS, DirUtils.getAxis(meta & 3)).func_177226_a(ACTIVE, meta >> 2 == 1);
    }

    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Nonnull
    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos) {
        switch (state.func_177229_b(AXIS)) {
            case X:
                return X_AABB;
            case Y:
                return Y_AABB;
            case Z:
            default:
                return Z_AABB;
        }
    }

    @Override
    public int func_149738_a(World world) {
        return 20;
    }

    public ItemStack getAxle(IBlockAccess world, BlockPos pos, IBlockState state) {
        return new ItemStack(BWMBlocks.WOODEN_AXLE);
    }

    @Override
    public void getDrops(@Nonnull NonNullList<ItemStack> drops, IBlockAccess world, BlockPos pos, @Nonnull IBlockState state, int fortune) {
        super.getDrops(drops, world, pos, state, fortune);
        drops.add(getAxle(world, pos, state));
        drops.add(func_185473_a((World) world, pos, state));

    }

    public IBlockState getAxisState(EnumFacing.Axis axis) {
        return this.func_176223_P().func_177226_a(AXIS, axis);
    }

    public EnumFacing.Axis getAxis(IBlockState state) {
        return state.func_177229_b(AXIS);
    }

    @Override
    public boolean canPlaceTorchOnTop(IBlockState state, @Nonnull IBlockAccess world, @Nonnull BlockPos pos) {
        return getAxis(state) == Y;
    }

    @Nonnull
    @Override
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face) {
        return face.func_176740_k() == getAxis(state) ? BlockFaceShape.CENTER : BlockFaceShape.UNDEFINED;
    }
}

