package betterwithmods.module.gameplay;

import betterwithmods.BWMod;
import betterwithmods.api.recipe.impl.RandomCountOutputs;
import betterwithmods.api.recipe.impl.RandomOutput;
import betterwithmods.api.util.IBlockVariants;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.BWMRecipes;
import betterwithmods.common.BWOreDictionary;
import betterwithmods.common.BWRegistry;
import betterwithmods.common.registry.block.recipe.BlockDropIngredient;
import betterwithmods.common.registry.block.recipe.BlockIngredient;
import betterwithmods.common.registry.block.recipe.SawRecipe;
import betterwithmods.common.registry.crafting.ChoppingRecipe;
import betterwithmods.module.Feature;
import betterwithmods.module.hardcore.crafting.HCLumber;
import com.google.common.collect.Lists;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPostInitializationEvent;
import net.minecraftforge.oredict.OreDictionary;

/**
 * Created by primetoxinz on 5/16/17.
 */
public class SawRecipes extends Feature {
    private static int plankCount, barkCount, sawDustCount;

    public SawRecipes() {
        canDisable = false;
    }

    @Override
    public void setupConfig() {
        //TODO make the default 4 in 1.13
        plankCount = loadPropInt("Saw Plank Output", "Plank count that is output when a log is chopped by a Saw.", 4);
        barkCount = loadPropInt("Saw Bark Output", "Bark count that is output when a log is chopped by a Saw.", 1);
        sawDustCount = loadPropInt("Saw sawdust Output", "Sawdust count that is output when a log is chopped by a Saw.", 2);
    }

    @Override
    public void init(FMLInitializationEvent event) {
        BWRegistry.WOOD_SAW.addSelfdropRecipe(new ItemStack(Blocks.field_150423_aK, 0, OreDictionary.WILDCARD_VALUE));
        BWRegistry.WOOD_SAW.addSelfdropRecipe(new ItemStack(Blocks.field_150395_bd));
        BWRegistry.WOOD_SAW.addSelfdropRecipe(new ItemStack(Blocks.field_150327_N));
        BWRegistry.WOOD_SAW.addSelfdropRecipe(new ItemStack(Blocks.field_150338_P));
        BWRegistry.WOOD_SAW.addSelfdropRecipe(new ItemStack(Blocks.field_150337_Q));
        BWRegistry.WOOD_SAW.addSelfdropRecipe(new ItemStack(BWMBlocks.ROPE));
        for (int i = 0; i < 9; i++)
            BWRegistry.WOOD_SAW.addSelfdropRecipe(new ItemStack(Blocks.field_150328_O, 1, i));

        BWRegistry.WOOD_SAW.addRecipe(new SawRecipe(new BlockIngredient(new ItemStack(Blocks.field_150440_ba)), new RandomCountOutputs(new RandomOutput(new ItemStack(Items.field_151127_ba), 3, 8))));

        BWOreDictionary.findLogRecipes();
        //TODO configure this
        BWOreDictionary.logRecipes.forEach(BWMRecipes::removeRecipe);
        int plankCount = BWMod.MODULE_LOADER.isFeatureEnabled(HCLumber.class) ? HCLumber.axePlankAmount: 4;
        if (!Loader.isModLoaded("primal")) {
            for(IBlockVariants variant: BWOreDictionary.blockVariants) {
                ItemStack log = variant.getVariant(IBlockVariants.EnumBlock.LOG,1);
                if(!log.func_190926_b()) {
                    ResourceLocation location = new ResourceLocation(BWMod.MODID, log.func_77973_b().getRegistryName().func_110623_a() + "_" + log.func_77960_j());
                    BWMRecipes.addRecipe(new ChoppingRecipe(variant, plankCount).setRegistryName(location));
                }
            }
        }
    }

    @Override
    public void postInit(FMLPostInitializationEvent event) {
        int count = BWMod.MODULE_LOADER.isFeatureEnabled(HCLumber.class) ? 4 : 6;
        for (IBlockVariants wood : BWOreDictionary.blockVariants) {
            BWRegistry.WOOD_SAW.addRecipe(new BlockDropIngredient(wood.getVariant(IBlockVariants.EnumBlock.LOG, 1)), Lists.newArrayList(wood.getVariant(IBlockVariants.EnumBlock.BLOCK, count), wood.getVariant(IBlockVariants.EnumBlock.BARK, 1), wood.getVariant(IBlockVariants.EnumBlock.SAWDUST, 2)));
        }
    }
    @Override
    public boolean hasSubscriptions() {
        return true;
    }
}
