package betterwithmods.module.gameplay.breeding_harness;

import betterwithmods.BWMod;
import betterwithmods.common.BWMItems;
import betterwithmods.common.items.ItemBreedingHarness;
import betterwithmods.module.Feature;
import betterwithmods.network.BWNetwork;
import betterwithmods.network.messages.MessageHarness;
import betterwithmods.util.InvUtils;
import com.google.common.collect.Sets;
import it.unimi.dsi.fastutil.objects.Object2BooleanMap;
import it.unimi.dsi.fastutil.objects.Object2BooleanOpenHashMap;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.passive.EntityAnimal;
import net.minecraft.entity.passive.EntityCow;
import net.minecraft.entity.passive.EntityPig;
import net.minecraft.entity.passive.EntitySheep;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumHand;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundCategory;
import net.minecraftforge.common.capabilities.CapabilityManager;
import net.minecraftforge.event.AttachCapabilitiesEvent;
import net.minecraftforge.event.entity.living.LivingEvent;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.items.ItemHandlerHelper;

import java.util.Set;

public class BreedingHarness extends Feature {

    public static final Item BREEDING_HARNESS = new ItemBreedingHarness().setRegistryName("breeding_harness");

    public BreedingHarness() {
    }

    @Override
    public void preInit(FMLPreInitializationEvent event) {
        BWMItems.registerItem(BREEDING_HARNESS);
        CapabilityManager.INSTANCE.register(CapabilityHarness.class, new CapabilityHarness.Storage(), CapabilityHarness::new);
    }

    private static final ResourceLocation CAPABILITY = new ResourceLocation(BWMod.MODID, "harness");

    private static void sendPacket(Entity entity) {
        CapabilityHarness cap = getCapability(entity);
        if (cap != null) {
            BWNetwork.sendToAllAround(new MessageHarness(entity.func_145782_y(), cap.getHarness()), entity.func_130014_f_(),entity.func_180425_c());
        }
    }

    @SubscribeEvent
    public void onAttach(AttachCapabilitiesEvent<Entity> event) {
        Entity entity = event.getObject();
        if (harnessEntity(entity)) {
            event.addCapability(CAPABILITY, new CapabilityHarness());
        }
    }

    @SubscribeEvent
    public void onTrack(PlayerEvent.StartTracking event) {
        if (event.getEntityPlayer().field_70170_p.field_72995_K)
            return;
        Entity entity = event.getTarget();
        sendPacket(entity);
    }

    @SubscribeEvent
    public void onEntityInteract(PlayerInteractEvent.EntityInteract event) {
        if (event.getWorld().field_72995_K)
            return;
        Entity entity = event.getTarget();

        CapabilityHarness cap = getCapability(entity);
        if (cap != null) {
            ItemStack hand = event.getItemStack();
            ItemStack harness = cap.getHarness();
            if (harness.func_190926_b() && !event.getEntityPlayer().func_70093_af()) {
                if (hand.func_77973_b() instanceof ItemBreedingHarness) {
                    cap.setHarness(InvUtils.setCount(hand.func_77946_l(), 1));
                    hand.func_190918_g(1);
                    event.getWorld().func_184133_a(null, entity.func_180425_c(), SoundEvents.field_187728_s, SoundCategory.NEUTRAL, 0.5f, 1.3f);
                    event.getWorld().func_184133_a(null, entity.func_180425_c(), SoundEvents.field_187713_n, SoundCategory.NEUTRAL, 0.5f, 1.3f);
                    event.getEntityPlayer().func_184609_a(EnumHand.MAIN_HAND);
                    if (entity instanceof EntitySheep) {
                        ((EntitySheep) entity).func_70893_e(true);
                    }
                    sendPacket(entity);
                }
            } else if (!harness.func_190926_b() && event.getEntityPlayer().func_70093_af() && hand.func_190926_b()) {
                ItemHandlerHelper.giveItemToPlayer(event.getEntityPlayer(), harness.func_77946_l());
                harness.func_190918_g(1);
                event.getWorld().func_184133_a(null, entity.func_180425_c(), SoundEvents.field_187728_s, SoundCategory.NEUTRAL, 1, 1);
                event.getWorld().func_184133_a(null, entity.func_180425_c(), SoundEvents.field_187713_n, SoundCategory.NEUTRAL, 1, 1f);
                event.getEntityPlayer().func_184609_a(EnumHand.MAIN_HAND);
                sendPacket(entity);
            }

        }

    }


    @SubscribeEvent
    public void onLivingTick(LivingEvent.LivingUpdateEvent e) {
        EntityLivingBase entity = e.getEntityLiving();
        if(harnessEntity(entity)) {
            if (hasHarness(entity)) {
                entity.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(-1);
                if (entity instanceof EntitySheep && !((EntitySheep) entity).func_70892_o()) {
                    ((EntitySheep) entity).func_70893_e(true);
                }
            } else {
                entity.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(0.25);
            }
        }
    }


    public static CapabilityHarness getCapability(Entity entity) {
        return entity.getCapability(CapabilityHarness.HARNESS_CAPABILITY, null);
    }

    public static boolean hasHarness(Entity entity) {
        CapabilityHarness cap = getCapability(entity);
        return cap != null && cap.getHarness().func_77973_b() instanceof ItemBreedingHarness;
    }

    public static final Set<Class<? extends EntityAnimal>> HARNESS_ANIMALS = Sets.newHashSet();
    private static final Object2BooleanMap<Class<? extends Entity>> HARNESS_CACHE = new Object2BooleanOpenHashMap<>();
    static {
        HARNESS_ANIMALS.add(EntityCow.class);
        HARNESS_ANIMALS.add(EntitySheep.class);
        HARNESS_ANIMALS.add(EntityPig.class);
        for (Class<? extends EntityAnimal> c : HARNESS_ANIMALS){
            HARNESS_CACHE.put(c, true);
        }
    }
    public static boolean harnessEntity(Entity entity) {
        if (!(entity instanceof EntityAnimal)){
            return false;
        }
        Class<? extends Entity> c = entity.getClass();
        Boolean harness = HARNESS_CACHE.get(c);
        if (harness != null){
            return harness;
        }
        boolean canHarness = false;
        for (Class<? extends EntityAnimal> cTest : HARNESS_ANIMALS){
            if (cTest.isAssignableFrom(c)){
                canHarness = true;
                break;
            }
        }
        return HARNESS_CACHE.put(c, canHarness);
    }

    @Override
    public boolean hasSubscriptions() {
        return true;
    }
}
