package betterwithmods.module.gameplay.miniblocks.blocks;

import betterwithmods.common.blocks.camo.BlockCamo;
import betterwithmods.common.blocks.camo.TileCamo;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Lists;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.property.ExtendedBlockState;
import net.minecraftforge.common.property.IUnlistedProperty;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collection;
import java.util.List;
import java.util.function.Function;

public abstract class BlockFurniture extends BlockCamo {
    protected static final PropertyBool SUPPORTED = PropertyBool.func_177716_a("supported");

    public BlockFurniture(Material material, Function<Material, Collection<IBlockState>> subtypes) {
        super(material, subtypes);
    }

    public static RayTraceResult raytraceMultiAABB(List<AxisAlignedBB> aabbs, BlockPos pos, Vec3d start, Vec3d end) {
        List<RayTraceResult> list = Lists.newArrayList();

        for (AxisAlignedBB axisalignedbb : aabbs) {
            list.add(rayTrace2(pos, start, end, axisalignedbb));
        }

        RayTraceResult raytraceresult1 = null;
        double d1 = 0.0D;

        for (RayTraceResult raytraceresult : list) {
            if (raytraceresult != null) {
                double d0 = raytraceresult.field_72307_f.func_72436_e(end);

                if (d0 > d1) {
                    raytraceresult1 = raytraceresult;
                    d1 = d0;
                }
            }
        }

        return raytraceresult1;
    }

    // Block.raytrace
    private static RayTraceResult rayTrace2(BlockPos pos, Vec3d start, Vec3d end, AxisAlignedBB boundingBox) {
        Vec3d vec3d = start.func_178786_a((double) pos.func_177958_n(), (double) pos.func_177956_o(), (double) pos.func_177952_p());
        Vec3d vec3d1 = end.func_178786_a((double) pos.func_177958_n(), (double) pos.func_177956_o(), (double) pos.func_177952_p());
        RayTraceResult raytraceresult = boundingBox.func_72327_a(vec3d, vec3d1);
        return raytraceresult == null ? null : new RayTraceResult(raytraceresult.field_72307_f.func_72441_c((double) pos.func_177958_n(), (double) pos.func_177956_o(), (double) pos.func_177952_p()), raytraceresult.field_178784_b, pos);
    }

    @Nullable
    @Override
    public TileEntity createTileEntity(@Nonnull World world, @Nonnull IBlockState state) {
        return new TileCamo();
    }

    @Nonnull
    @Override
    protected BlockStateContainer func_180661_e() {
        return new ExtendedBlockState(this, new IProperty[]{SUPPORTED}, new IUnlistedProperty[]{CAMO_INFO});
    }

    @Override
    public boolean isSideSolid(IBlockState base_state, @Nonnull IBlockAccess world, @Nonnull BlockPos pos, EnumFacing side) {
        return false;
    }

    @Nonnull
    @Override
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face) {
        return face == EnumFacing.DOWN ? BlockFaceShape.CENTER : BlockFaceShape.UNDEFINED;
    }

    @Nonnull
    @Override
    public IBlockState func_176221_a(@Nonnull IBlockState state, IBlockAccess world, BlockPos pos) {
        boolean connected = isConnected(state, world, pos, EnumFacing.NORTH) || isConnected(state, world, pos, EnumFacing.WEST);
        return state.func_177226_a(SUPPORTED, !connected);
    }

    private boolean isConnected(IBlockState state, IBlockAccess world, BlockPos pos, EnumFacing facing) {
        EnumFacing opp = facing.func_176734_d();
        IBlockState state1 = world.func_180495_p(pos.func_177972_a(facing));
        IBlockState state2 = world.func_180495_p(pos.func_177972_a(opp));
        return state1.func_177230_c().canBeConnectedTo(world, pos.func_177972_a(facing), opp) && state2.func_177230_c().canBeConnectedTo(world, pos.func_177972_a(opp), facing);
    }

    @Override
    public abstract boolean canBeConnectedTo(IBlockAccess world, BlockPos pos, EnumFacing facing);

    public double getOffset() {
        return 0;
    }

    @Override
    public RayTraceResult func_180636_a(IBlockState blockState, @Nonnull World worldIn, @Nonnull BlockPos pos, @Nonnull Vec3d start, @Nonnull Vec3d end) {
        return raytraceMultiAABB(getBoxes(), pos, start, end);
    }

    public abstract ImmutableList<AxisAlignedBB> getBoxes();

}
