package betterwithmods.module.gameplay.miniblocks.orientations;

import net.minecraft.client.renderer.block.model.ModelRotation;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.Vec3d;
import net.minecraftforge.common.model.TRSRTransformation;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nonnull;

import static betterwithmods.module.gameplay.miniblocks.orientations.OrientationUtils.inCenter;
import static betterwithmods.module.gameplay.miniblocks.orientations.OrientationUtils.isMax;

public enum ColumnOrientation implements BaseOrientation {
    DOWN("down", EnumFacing.DOWN, 0, 0, new AxisAlignedBB(2 / 16d, 0.0D, 2 / 16d, 14 / 16d, 1, 14 / 16d));

    public static final ColumnOrientation[] VALUES = values();

    private final String name;
    private final EnumFacing facing;
    private final AxisAlignedBB bounds;
    private final int x;
    private final int y;

    ColumnOrientation(String name, EnumFacing facing, int x, int y, AxisAlignedBB bounds) {
        this.name = name;
        this.facing = facing;
        this.x = x;
        this.y = y;
        this.bounds = bounds;
    }

    public static BaseOrientation fromFace(EnumFacing facing) {
        if (facing != null)
            return ColumnOrientation.VALUES[facing.func_176745_a()];
        return BaseOrientation.DEFAULT;
    }

    public static BaseOrientation getFromVec(Vec3d hit, EnumFacing facing) {
        float hitXFromCenter = (float) (hit.field_72450_a - 0.5F);
        float hitYFromCenter = (float) (hit.field_72448_b - 0.5F);
        float hitZFromCenter = (float) (hit.field_72449_c - 0.5F);
        switch (facing.func_176740_k()) {
            case Y:
                if (inCenter(hitXFromCenter, hitZFromCenter, 0.25f)) {
                    return fromFace(facing);
                } else if (isMax(hitXFromCenter, hitZFromCenter)) {
                    return hitXFromCenter < 0 ? fromFace(EnumFacing.EAST) : fromFace(EnumFacing.WEST);
                } else {
                    return hitZFromCenter < 0 ? fromFace(EnumFacing.SOUTH) : fromFace(EnumFacing.NORTH);
                }
            case X:
                if (inCenter(hitYFromCenter, hitZFromCenter, 0.25f)) {
                    return fromFace(facing);
                } else if (isMax(hitYFromCenter, hitZFromCenter)) {
                    return hitYFromCenter < 0 ? fromFace(EnumFacing.UP) : fromFace(EnumFacing.DOWN);

                } else {
                    return hitZFromCenter < 0 ? fromFace(EnumFacing.SOUTH) : fromFace(EnumFacing.NORTH);
                }
            case Z:
                if (inCenter(hitYFromCenter, hitXFromCenter, 0.25f)) {
                    return fromFace(facing);
                } else if (isMax(hitYFromCenter, hitXFromCenter)) {
                    return hitYFromCenter < 0 ? fromFace(EnumFacing.UP) : fromFace(EnumFacing.DOWN);
                } else {
                    return hitXFromCenter < 0 ? fromFace(EnumFacing.EAST) : fromFace(EnumFacing.WEST);
                }
            default:
                return fromFace(facing);
        }
    }

    @Nonnull
    @Override
    public String func_176610_l() {
        return name;
    }

    public EnumFacing getFacing() {
        return facing;
    }

    @Override
    public AxisAlignedBB getBounds() {
        return bounds;
    }

    @SideOnly(Side.CLIENT)
    @Override
    public TRSRTransformation toTransformation() {
        return TRSRTransformation.from(ModelRotation.func_177524_a(x, y));
    }

    @Override
    public BaseOrientation next() {
        return VALUES[(this.ordinal() + 1) % (VALUES.length)];
    }
}

