package betterwithmods.testing.base;

import betterwithmods.common.registry.block.managers.CraftingManagerBlock;
import betterwithmods.common.registry.block.recipe.BlockRecipe;
import betterwithmods.testing.base.world.FakeWorld;
import net.minecraft.init.Blocks;
import net.minecraft.util.math.BlockPos;
import org.fest.assertions.Assertions;

import java.util.Optional;


public abstract class BaseBlockTest<T extends BlockRecipe> extends BaseTest {

    protected final BlockPos origin = BlockPos.field_177992_a;
    protected FakeWorld world;
    protected CraftingManagerBlock<T> TEST_MANAGER;
    protected T recipe, oreRecipe, blockDropRecipe;

    public abstract void beforeTest();

    @Test
    public void testFakeWorld() {
        world.func_175698_g(origin);
        Assertions.assertThat(world.func_180495_p(origin)).isEqualTo(Blocks.field_150350_a.func_176223_P());
        world.func_175656_a(origin, Blocks.field_150348_b.func_176223_P());
        Assertions.assertThat(world.func_180495_p(origin)).isEqualTo(Blocks.field_150348_b.func_176223_P());
    }

    @Test
    public void testRecipeAddition() {
        Assertions.assertThat(recipe.isInvalid()).isFalse();
        Assertions.assertThat(TEST_MANAGER.getRecipes()).isEmpty();
        TEST_MANAGER.addRecipe(recipe);
        Assertions.assertThat(TEST_MANAGER.getRecipes()).hasSize(1);
    }

    @Test
    public void testRecipeRemoval() {
        Assertions.assertThat(recipe.isInvalid()).isFalse();
        Assertions.assertThat(TEST_MANAGER.getRecipes()).isEmpty();
        TEST_MANAGER.addRecipe(recipe);
        Assertions.assertThat(TEST_MANAGER.getRecipes()).isNotEmpty();
        TEST_MANAGER.removeRecipe(recipe);
    }

    @Test
    public void testRecipeMatching() {
        testRecipe(recipe, origin);
    }

    @Test
    public void testOredictMatching() {
        testRecipe(oreRecipe, origin);
    }

    @Test
    public void testBlockDropMatching() {
        testRecipe(blockDropRecipe, origin.func_177984_a());
    }


    @Test
    public void testPriority() {

    }

    protected void testRecipe(T recipe, BlockPos pos) {
        Assertions.assertThat(recipe.isInvalid()).isFalse();
        Assertions.assertThat(TEST_MANAGER.getRecipes()).isEmpty();
        TEST_MANAGER.addRecipe(recipe);
        Optional<T> foundRecipe = TEST_MANAGER.findRecipe(world,origin, world.func_180495_p(pos));
        Assertions.assertThat(foundRecipe.isPresent()).isTrue();
        if(foundRecipe.isPresent()) {
            T actualRecipe = foundRecipe.get();
            Assertions.assertThat(actualRecipe.craftRecipe(world, origin, world.field_73012_v, world.func_180495_p(pos))).isTrue();
            TEST_MANAGER.getRecipes().clear();
        }
    }
}

